import json
import logging
from pathlib import Path, PurePath

import click
import pandas as pd

from src.data import read_csv, split_train_val_data
from src.entities import TotalPipelineParams, read_training_pipeline_params
from src.features import PreprocessingTransformer, extract_target
from src.models import (train_model, predict_model, load_model, save_model, evaluate_model)

logging.basicConfig(
    filename=PurePath(Path(__file__).parents[0], Path(f'logs/{__name__}.log')),
    format='%(asctime)s,%(msecs)d %(levelname)-8s [%(filename)s:%(lineno)d] %(message)s',
    datefmt='%Y-%m-%d:%H:%M:%S',
    level=logging.DEBUG,
    filemode='w')
logger = logging.getLogger(__name__)


def run_pipeline(config_path: str, process_type: str) -> None:
    logger.debug("In function src.model_pipeline.run_pipeline")
    params = read_training_pipeline_params(config_path)
    df = read_csv(params.dataset_params.input_file_path)

    train_df, val_df = split_train_val_data(df, params.splitting_params, params.dataset_params)

    train_df, train_target = extract_target(train_df, params.feature_params)
    val_df, val_target = extract_target(val_df, params.feature_params)

    preprocess = PreprocessingTransformer(params.preprocessing_params, params.feature_params)
    if process_type == 'train':
        logger.info("Start train")
        train_df = pd.DataFrame(preprocess.fit_transform(train_df))
        model = train_model(train_df, train_target, params.model_params)
        save_model(model, params.model_params)
    if process_type == 'predict':
        logger.info("Start predict")
        val_df = pd.DataFrame(preprocess.fit_transform(val_df))
        model = load_model(params.model_params)
        predicted_val_target = predict_model(model, val_df)
        results = evaluate_model(predicted_val_target, val_target)

        with open('results/metrics.json', 'w') as f:
            print(results)
            json.dump(results, f)
        logger.info("Results saved to results/metrics.json")
    logger.debug("Out of function src.model_pipeline.run_pipeline")


@click.command(name="pipeline")
@click.option('--process-type', type=click.Choice(['train', 'predict'], case_sensitive=False))
@click.argument("config_path")
def model_pipe(process_type: str, config_path: str):
    logger.info("Start pipeline with parameters: precess_type=%s, config_path=%s", process_type, config_path)
    run_pipeline(config_path, process_type)


if __name__ == "__main__":
    model_pipe()



"""Unittest for src.data.entities.*"""

import unittest
from unittest import mock
import logging
import pandas as pd

import src
from src.entities import *


class TestDataClass(unittest.TestCase):
    """Tests for dataclasses"""

    def test_dataset_params(self):
        """Should test for DatasetParams"""
        input_file = 'test.csv'
        output_files = ['test1.csv', 'test2.csv']

        ds = DatasetParams(input_file_path=input_file, output_paths=output_files)
        self.assertEqual(ds.input_file_path, input_file)
        self.assertEqual(ds.output_paths, output_files)

    def test_feature_params(self):
        """Should test for FeatureParams"""
        cat_cols = ['1', '2', '3']
        num_cols = ['4', '5', '6']
        target_col = '7'

        ft = FeatureParams(categorical_features=cat_cols, numerical_features=num_cols, target_col=target_col)
        self.assertEqual(ft.target_col, target_col)
        self.assertEqual(ft.categorical_features, cat_cols)
        self.assertEqual(ft.numerical_features, num_cols)

    def test_preprocessing_params(self):
        """Should test for PreprocessingParams"""
        params_list = ['1', '2', '3']
        pp = PreprocessingParams(pipeline=params_list)
        self.assertEqual(pp.pipeline, params_list)

    def test_model_params(self):
        """Should test for ModelParams"""
        model_save_path = 'test.csv'
        model_name = 'Model'
        parameters = [1, 2, 3]
        sp = ModelParams(model_save_path=model_save_path, model_name=model_name, parameters=parameters)
        self.assertEqual(sp.model_save_path, model_save_path)
        self.assertEqual(sp.model_name, model_name)
        self.assertEqual(sp.parameters, parameters)


if __name__ == "__main__":
    unittest.main()



"""Unittest for src.model_pipeline"""

import logging
import unittest
from io import StringIO
from click.testing import CliRunner
from src.model_pipeline import model_pipe
import src


# switch off import-logging
logging.getLogger(src.model_pipeline.__name__).disabled = True


class TestPipeline(unittest.TestCase):
    """Tests for main pipeline"""
    def test_click(self):
        """Should test click"""
        runner = CliRunner()
        result = runner.invoke(model_pipe, ['--process-type=train', 'configs/logistic_regression_config.yaml'])
        assert result.exit_code == 0


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
from distutils.core import setup


def readme():
    """Import the README.md Markdown file and try to convert it to RST format."""
    try:
        import pypandoc
        return pypandoc.convert('README.md', 'rst')
    except(IOError, ImportError):
        with open('README.md') as readme_file:
            return readme_file.read()


def required():
    """Create list of requires to setup function"""
    with open('requirements.txt') as f:
        rq = f.read().splitlines()
    return rq


setup(
    name='Mlops-ml-deploy-made-iv',
    version='0.3',
    description='Mlops-ml-deploy-made-iv',
    long_description=readme(),
    url='https://github.com/Z5-05/mlops_made_2022/tree/hw01',
    author='iverendeev',
    install_requires=required(),
    author_email='ilaverendeev@gmail.com',
    license='MIT',
    packages=['src', 'tests'],
)
